import pickle
import logging
import copy
import pandas as pd
from alphad3m.primitive_loader import load_primitives_by_name, load_primitives_by_id, load_primitives_types
from alphad3m.metalearning.resource_builder import get_dataset_id, filter_primitives, load_precalculated_data
from alphad3m.metalearning.dataset_profiler import DEFAULT_METAFEATURES

logger = logging.getLogger(__name__)


IGNORE_PRIMITIVES = {
    # These primitives are static elements in the pipelines, not considered as part of the pattern
    'd3m.primitives.data_transformation.construct_predictions.Common',
    'd3m.primitives.data_transformation.extract_columns_by_semantic_types.Common',
    'd3m.primitives.data_transformation.dataset_to_dataframe.Common',
    'd3m.primitives.data_transformation.denormalize.Common',
    'd3m.primitives.data_transformation.flatten.DataFrameCommon',
    'd3m.primitives.data_transformation.column_parser.Common',
    'd3m.primitives.data_transformation.simple_column_parser.DataFrameCommon',
    'd3m.primitives.data_transformation.do_nothing.DSBOX',
    'd3m.primitives.data_transformation.do_nothing_for_dataset.DSBOX',
    'd3m.primitives.data_transformation.add_semantic_types.Common',
    'd3m.primitives.schema_discovery.profiler.Common',
    'd3m.primitives.schema_discovery.profiler.DSBOX',
    'd3m.primitives.data_cleaning.column_type_profiler.Simon',
    'd3m.primitives.operator.compute_unique_values.Common',
    'd3m.primitives.data_transformation.construct_confidence.Common',
    # We add these primitives internally because they require special connections
    'd3m.primitives.data_transformation.text_reader.Common',
    'd3m.primitives.data_transformation.image_reader.Common'
}


def create_csv_data(metalearningdb_pickle_path, pipelines_csv_path,  use_primitive_names=True):
    logger.info('Creating CSV file...')
    primitives_by_name = load_primitives_by_name(only_installed_primitives=False)
    primitives_by_id = load_primitives_by_id(only_installed_primitives=False)
    primitives_types = load_primitives_types(only_installed_primitives=False)
    dataset_task_keywords = load_precalculated_data('task_keywords')
    dataset_semantic_types = load_precalculated_data('dataprofiles')
    dataset_metafeatures = load_precalculated_data('metafeatures')
    ignore_primitives_ids = {primitives_by_name[ignore_primitive]['id'] for ignore_primitive in IGNORE_PRIMITIVES}
    train_pipelines = []
    total_pipelines = 0

    with open(metalearningdb_pickle_path, 'rb') as fin:
        all_pipelines = pickle.load(fin)

    for pipeline_run in all_pipelines:
        dataset_id = get_dataset_id(pipeline_run['problem']['id'])

        if dataset_id not in dataset_task_keywords:
            continue

        pipeline_primitives = pipeline_run['steps']
        pipeline_primitives = filter_primitives(pipeline_primitives, ignore_primitives_ids)  # Get the IDs of primitives
        task_keywords = ' '.join(dataset_task_keywords[dataset_id]['task_keywords'])
        semantic_types = ' '.join(dataset_semantic_types[dataset_id]['feature_types'])
        metafeatures = [dataset_metafeatures[dataset_id][mf]['value'] for mf in DEFAULT_METAFEATURES]

        if len(pipeline_primitives) > 0:
            score = pipeline_run['scores'][0]['normalized']
            metric = pipeline_run['scores'][0]['metric']['metric']
            try:
                pipeline_primitive_types = [primitives_types[primitives_by_id[p]] for p in pipeline_primitives]
                if use_primitive_names:
                    pipeline_primitives = [primitives_by_id[p] for p in pipeline_primitives]
                pipeline_stages = generate_pipeline_stages(pipeline_primitives, pipeline_primitive_types)
                pipeline_stages = [[' '.join(ps), task_keywords, semantic_types, metric, score] + metafeatures
                                   for ps in pipeline_stages]
                train_pipelines += pipeline_stages
                total_pipelines += 1
            except Exception:
                logger.warning(f'Primitives "{str(pipeline_primitives)}" are not longer available')

    logger.info(f'Loaded {len(all_pipelines)} pipelines')
    logger.info(f'Found {total_pipelines} pipelines')
    pipelines_df = pd.DataFrame.from_records(train_pipelines, columns=['primitives', 'task_keywords', 'semantic_types',
                                                                       'metric', 'score'] + DEFAULT_METAFEATURES)
    pipelines_df.to_csv(pipelines_csv_path, index=False)


def generate_pipeline_stages(primitive_items, primitive_types):
    combinations = []
    pre_pipeline = primitive_types
    combinations.append(copy.deepcopy(pre_pipeline))

    for index, primitive_item in enumerate(primitive_items):
        pre_pipeline[index] = primitive_item
        combinations.append(copy.deepcopy(pre_pipeline))

    return combinations


if __name__ == '__main__':
    # Download the metalearningdb.pkl file from https://drive.google.com/file/d/1WjY7iKkkKMZFeoiCqzamA_iqVOwQidXS/view
    # and D3M datasets from https://datasets.datadrivendiscovery.org/d3m/datasets/-/tree/master/seed_datasets_current
    metalearningdb_pickle_path = '/Users/rlopez/D3M/metalearning_db/metalearningdb.pkl'
    pipelines_csv_path = '/Users/rlopez/D3M/metalearning_db/marvin_pipelines.csv'

    create_csv_data(metalearningdb_pickle_path, pipelines_csv_path)



import logging
import math
import hashlib
import datamart_profiler
from metalearn import Metafeatures
from alphad3m.metalearning.resource_builder import load_precalculated_data
from alphad3m.metalearning.dataset_profiler import extract_dataprofiles, extract_metafeatures
from sklearn.metrics.pairwise import cosine_similarity
from d3m.metadata.problem import TaskKeywordBase

logger = logging.getLogger(__name__)


def create_metafeatures_vector(metafeatures, metafeature_indices):
    vector = []

    for metafeatures_id in metafeature_indices:
        value = metafeatures[metafeatures_id]['value']
        if isinstance(value, str):
            value = int(hashlib.sha256(value.encode('utf-8')).hexdigest(), 16) % 256
        elif math.isnan(value) or math.isinf(value):
            value = 0
        vector.append(value)

    return vector


def create_dataprofiles_vector(dataprofiles, dataprofile_indices):
    vector = []

    for dataprofile_id in dataprofile_indices:
        if dataprofile_id in dataprofiles['feature_types']:
            value = 1
        else:
            value = 0
        vector.append(value)

    value = 1 if dataprofiles['missing_values'] else 0
    vector.append(value)  # Add an extra value corresponding to the missing values data

    return vector


def create_taskkeywords_vector(task_keywords, taskkeyword_indices):
    vector = []

    for taskkeyword_id in taskkeyword_indices:
        if taskkeyword_id in task_keywords['task_keywords']:
            value = 1
        else:
            value = 0
        vector.append(value)

    return vector


def create_metafeatures_vectors_mldb(metafeature_indices):
    vectors = {}
    metafeature_datasets = load_precalculated_data('metafeatures')

    for id_dataset, metafeatures in metafeature_datasets.items():
        vector = create_metafeatures_vector(metafeatures, metafeature_indices)
        vectors[id_dataset] = vector

    return vectors


def create_dataprofiles_vectors_mldb(dataprofile_indices):
    vectors = {}
    dataprofile_datasets = load_precalculated_data('dataprofiles')

    for id_dataset, dataprofiles in dataprofile_datasets.items():
        vector = create_dataprofiles_vector(dataprofiles, dataprofile_indices)
        vectors[id_dataset] = vector

    return vectors


def create_taskkeywords_vectors_mldb(taskkeyword_indices):
    vectors = {}
    taskkeyword_datasets = load_precalculated_data('task_keywords')

    for id_dataset, task_keywords in taskkeyword_datasets.items():
        vector = create_taskkeywords_vector(task_keywords, taskkeyword_indices)
        vectors[id_dataset] = vector

    return vectors


def load_metafeatures_vectors(dataset_path, target_column):
    mfs = extract_metafeatures(dataset_path, target_column)
    metafeature_indices = Metafeatures.list_metafeatures(group='all')
    target_metafeatures_vector = create_metafeatures_vector(mfs, metafeature_indices)
    metalearningdb_vectors = create_metafeatures_vectors_mldb(metafeature_indices)

    return metalearningdb_vectors, target_metafeatures_vector


def load_profiles_vectors(dataset_path, target_column):
    dps = extract_dataprofiles(dataset_path, target_column)
    dataprofile_indices = [v for k, v in datamart_profiler.types.__dict__.items() if not k.startswith('_')]
    target_dataprofile_vector = create_dataprofiles_vector(dps, dataprofile_indices)
    metalearningdb_vectors = create_dataprofiles_vectors_mldb(dataprofile_indices)

    return metalearningdb_vectors, target_dataprofile_vector


def load_taskkeyword_vectors(task_keywords):
    taskkeyword_indices = sorted([keyword for keyword in TaskKeywordBase.get_map().keys() if keyword is not None])
    target_dataprofile_vector = create_taskkeywords_vector({'task_keywords': task_keywords}, taskkeyword_indices)
    metalearningdb_vectors = create_taskkeywords_vectors_mldb(taskkeyword_indices)

    return metalearningdb_vectors, target_dataprofile_vector


def calculate_similarity(metalearningdb_vectors, target_vector, threshold):
    similar_datasets = {}
    for id_dataset, vector in metalearningdb_vectors.items():
        similarity = round(cosine_similarity([target_vector], [vector]).flat[0], 5)
        if similarity >= threshold:
            similar_datasets[id_dataset] = similarity

    return similar_datasets


def similarity_repr(dataset_similarities):
    similarity_string = []

    for dataset_similarity in sorted(dataset_similarities.items(), key=lambda x: x[1], reverse=True):
        pretty_string = '%s=%.2f' % dataset_similarity
        similarity_string.append(pretty_string)

    return ', '.join(similarity_string)


def get_similar_datasets(mode, dataset_path, target_column, task_keywords, threshold=0.8, combined=False):
    vectors_taskkeywords, target_vector_taskkeywords = load_taskkeyword_vectors(task_keywords)

    if mode == 'metafeatures':
        vectors_dataset, target_vector_dataset = load_metafeatures_vectors(dataset_path, target_column)
    elif mode == 'dataprofiles':
        vectors_dataset, target_vector_dataset = load_profiles_vectors(dataset_path, target_column)
    else:
        raise ValueError('Unknown mode "%s" to load data' % mode)

    if combined:
        # Concatenate the vectors of the dataset and task keywords
        for id_dataset in vectors_dataset:
            vectors_dataset[id_dataset] += vectors_taskkeywords[id_dataset]
        target_vector_dataset += target_vector_taskkeywords
        similar_datasets = calculate_similarity(vectors_dataset, target_vector_dataset, threshold)
        logger.info('Similar datasets found using both information:\n%s', similarity_repr(similar_datasets))
    else:
        # Use threshold=1.0 to get datasets with the same task keywords
        similar_datasets = calculate_similarity(vectors_taskkeywords, target_vector_taskkeywords, 1.0)
        logger.info('Similar datasets found using task_keywords features:\n%s', similarity_repr(similar_datasets))
        vectors_dataset = {k: vectors_dataset[k] for k in similar_datasets}  # Use only the similar datasets
        similar_datasets = calculate_similarity(vectors_dataset, target_vector_dataset, threshold)
        logger.info('Similar datasets found using %s features:\n%s', mode, similarity_repr(similar_datasets))

    return similar_datasets



import re
import ast
import json
import gzip
import pickle
import logging
from os.path import join, dirname, exists
from alphad3m.primitive_loader import load_primitives_by_name
from alphad3m.metalearning.dataset_profiler import extract_dataprofiles, extract_metafeatures

logger = logging.getLogger(__name__)

METALEARNING_DB_PATH = join(dirname(__file__), '../resource/metalearning_db.json.gz')
PRECALCULATED_TASKKEYWORDS_PATH = join(dirname(__file__), '../resource/precalculated_taskkeywords.json')
PRECALCULATED_METAFEATURES_PATH = join(dirname(__file__), '../resource/precalculated_metafeatures.json')
PRECALCULATED_DATAPROFILES_PATH = join(dirname(__file__), '../resource/precalculated_dataprofiles.json')


IGNORE_PRIMITIVES = {
    # These primitives are static elements in the pipelines, not considered as part of the pattern
    'd3m.primitives.data_transformation.construct_predictions.Common',
    'd3m.primitives.data_transformation.extract_columns_by_semantic_types.Common',
    'd3m.primitives.data_transformation.dataset_to_dataframe.Common',
    'd3m.primitives.data_transformation.denormalize.Common',
    'd3m.primitives.data_transformation.flatten.DataFrameCommon',
    'd3m.primitives.data_transformation.column_parser.Common',
    'd3m.primitives.data_transformation.simple_column_parser.DataFrameCommon',
    'd3m.primitives.data_transformation.do_nothing.DSBOX',
    'd3m.primitives.data_transformation.do_nothing_for_dataset.DSBOX',
    'd3m.primitives.data_transformation.add_semantic_types.Common',
    'd3m.primitives.schema_discovery.profiler.Common',
    'd3m.primitives.schema_discovery.profiler.DSBOX',
    'd3m.primitives.data_cleaning.column_type_profiler.Simon',
    'd3m.primitives.operator.compute_unique_values.Common',
    'd3m.primitives.data_transformation.construct_confidence.Common',
    # We add these primitives internally because they require special connections
    'd3m.primitives.data_transformation.text_reader.Common',
    'd3m.primitives.data_transformation.image_reader.Common'
}


def create_compressed_metalearningdb(metalearningdb_pickle_path):
    logger.info('Compressing Meta-Learning DB...')
    primitives_by_name = load_primitives_by_name(only_installed_primitives=False)
    available_datasets = load_precalculated_data('task_keywords')
    ignore_primitives_ids = {primitives_by_name[ignore_primitive]['id'] for ignore_primitive in IGNORE_PRIMITIVES}
    pipelines_by_dataset = {}
    pipelines_hashing = {}

    with open(metalearningdb_pickle_path, 'rb') as fin:
        all_pipelines = pickle.load(fin)

    for pipeline_run in all_pipelines:
        dataset_id = get_dataset_id(pipeline_run['problem']['id'])

        if dataset_id not in available_datasets:
            continue

        pipeline_primitives = pipeline_run['steps']
        pipeline_primitives = filter_primitives(pipeline_primitives, ignore_primitives_ids)  # Get the IDs of primitives

        if dataset_id not in pipelines_by_dataset:
            pipelines_by_dataset[dataset_id] = {}

        if len(pipeline_primitives) > 0:
            score = pipeline_run['scores'][0]['normalized']
            metric = pipeline_run['scores'][0]['metric']['metric']
            pipeline_str = str(pipeline_primitives)

            if pipeline_str not in pipelines_hashing:
                hashing_value = 'P' + str(len(pipelines_hashing))
                pipelines_hashing[pipeline_str] = hashing_value

            pipeline_id = pipelines_hashing[pipeline_str]

            if pipeline_id not in pipelines_by_dataset[dataset_id]:
                pipelines_by_dataset[dataset_id][pipeline_id] = {'score': [], 'metric': []}

            pipelines_by_dataset[dataset_id][pipeline_id]['score'].append(score)
            pipelines_by_dataset[dataset_id][pipeline_id]['metric'].append(metric)

    pipeline_structure = {}
    for pipeline_str, pipeline_id in pipelines_hashing.items():
        primitives = [primitive for primitive in ast.literal_eval(pipeline_str)]  # Convert str to list
        pipeline_structure[pipeline_id] = primitives

    metalearning_db = {}
    metalearning_db['pipeline_performances'] = pipelines_by_dataset
    metalearning_db['pipeline_structure'] = pipeline_structure

    with gzip.open(METALEARNING_DB_PATH, 'wt', encoding='UTF-8') as zipfile:
        json.dump(json.dumps(metalearning_db), zipfile)  # Convert to str and then compress it

    logger.info('Compressing process ended')


def extract_taskkeywords_metalearningdb(datasets_path):
    datasets = get_unique_datasets()
    task_keywords = load_precalculated_data('task_keywords')

    for dataset_id in datasets:
        logger.info('Calculating task keywords for dataset %s...', dataset_id)
        if dataset_id not in task_keywords:
            try:
                _, _, keywords = load_task_info(dataset_id, datasets_path)
                task_keywords[dataset_id] = {'task_keywords': keywords}
                logger.info('Task keywords successfully calculated for dataset %s', dataset_id)
                with open(PRECALCULATED_TASKKEYWORDS_PATH, 'w') as fout:
                    json.dump(task_keywords, fout, indent=4, sort_keys=True)
            except Exception as e:
                logger.error(str(e))
        else:
            logger.info('Using pre-calculated task keywords for dataset %s', dataset_id)

    return task_keywords


def extract_metafeatures_metalearningdb(datasets_path):
    datasets = get_unique_datasets()
    metafeatures = load_precalculated_data('metafeatures')

    for dataset_id in datasets:
        logger.info('Calculating metafeatures for dataset %s...', dataset_id)
        if dataset_id not in metafeatures:
            try:
                dataset_path, target_column, _ = load_task_info(dataset_id, datasets_path, 'SCORE')
                mfs = extract_metafeatures(dataset_path, target_column)
                metafeatures[dataset_id] = mfs
                logger.info('Metafeatures successfully calculated for dataset %s', dataset_id)
                with open(PRECALCULATED_METAFEATURES_PATH, 'w') as fout:
                    json.dump(metafeatures, fout, indent=4, sort_keys=True)
            except Exception as e:
                logger.error(str(e))
        else:
            logger.info('Using pre-calculated metafeatures for dataset %s', dataset_id)

    return metafeatures


def extract_dataprofiles_metalearningdb(datasets_path):
    datasets = get_unique_datasets()
    dataprofiles = load_precalculated_data('dataprofiles')

    for dataset_id in datasets:
        logger.info('Calculating data profiles for dataset %s...', dataset_id)
        if dataset_id not in dataprofiles:
            try:
                dataset_path, target_column, _ = load_task_info(dataset_id, datasets_path)
                dps = extract_dataprofiles(dataset_path, target_column)
                dataprofiles[dataset_id] = dps
                logger.info('Data profiles successfully calculated for dataset %s', dataset_id)
                with open(PRECALCULATED_DATAPROFILES_PATH, 'w') as fout:
                    json.dump(dataprofiles, fout, indent=4, sort_keys=True)
            except Exception as e:
                logger.error(str(e))
        else:
            logger.info('Using pre-calculated data profiles for dataset %s', dataset_id)

    return dataprofiles


def load_task_info(dataset_id, datasets_path, suffix='TRAIN'):
    possible_names = [join(datasets_path, dataset_id), join(datasets_path, dataset_id + '_MIN_METADATA'),
                      join(datasets_path, dataset_id.replace('_MIN_METADATA', ''))]
    # All possible names of the datasets on disk, with/without the suffix 'MIN_METADATA'

    for dataset_folder_path in possible_names:
        if exists(dataset_folder_path):
            break
    else:
        raise FileNotFoundError('Dataset %s not found' % dataset_id)

    dataset_path = join(dataset_folder_path, suffix, 'dataset_%s/tables/learningData.csv' % suffix)
    problem_path = join(dataset_folder_path, suffix, 'problem_%s/problemDoc.json' % suffix)

    with open(problem_path) as fin:
        problem_doc = json.load(fin)
        task_keywords = problem_doc['about']['taskKeywords']
        target_column = problem_doc['inputs']['data'][0]['targets'][0]['colName']

    return dataset_path, target_column, task_keywords


def get_dataset_id(problem_id):
    # Remove suffixes 'TRAIN' and 'problem' from the dataset name
    dataset_id = re.sub('_TRAIN$', '', problem_id)
    dataset_id = re.sub('_problem$', '', dataset_id)

    return dataset_id


def get_unique_datasets():
    metalearning_db = load_metalearningdb()
    datasets = metalearning_db['pipeline_performances'].keys()

    return sorted(datasets)


def filter_primitives(pipeline_steps, ignore_primitives):
    primitives = []

    for pipeline_step in pipeline_steps:
        if pipeline_step['primitive']['id'] not in ignore_primitives:
            primitives.append(pipeline_step['primitive']['id'])

    if len(primitives) > 0 and primitives[0] == '7ddf2fd8-2f7f-4e53-96a7-0d9f5aeecf93':
        # Special case: Primitive to_numeric.DSBOX
        # This primitive should not be first because it only takes the numeric features, ignoring the remaining ones
        primitives = primitives[1:]

    return primitives


def load_metalearningdb():
    all_pipelines = []
    logger.info('Loading pipelines from metalearning database...')

    with gzip.open(METALEARNING_DB_PATH, 'rt', encoding='UTF-8') as zipfile:
        all_pipelines = json.loads(json.load(zipfile))  # Uncompress as str and then convert to dict

    logger.info('Found %d unique pipelines in metalearning database' % len(all_pipelines['pipeline_structure']))

    return all_pipelines


def load_precalculated_data(mode):
    if mode == 'metafeatures':
        file_path = PRECALCULATED_METAFEATURES_PATH
    elif mode == 'dataprofiles':
        file_path = PRECALCULATED_DATAPROFILES_PATH
    elif mode == 'task_keywords':
        file_path = PRECALCULATED_TASKKEYWORDS_PATH
    else:
        raise ValueError('Unknown mode "%s" to load data' % mode)

    if exists(file_path):
        with open(file_path) as fin:
            return json.load(fin)

    return {}


if __name__ == '__main__':
    # Run this to create the meta-learning DB, task keywords, data profiles, and meta-features files
    # Download the metalearningdb.pkl file from https://drive.google.com/file/d/1WjY7iKkkKMZFeoiCqzamA_iqVOwQidXS/view
    # and D3M datasets from https://datasets.datadrivendiscovery.org/d3m/datasets/-/tree/master/seed_datasets_current
    metalearningdb_pickle_path = '/Users/rlopez/D3M/metalearning_db/metalearningdb.pkl'
    datasets_root_path = '/Users/rlopez/D3M/datasets/seed_datasets_current/'

    create_compressed_metalearningdb(metalearningdb_pickle_path)
    extract_taskkeywords_metalearningdb(datasets_root_path)
    extract_dataprofiles_metalearningdb(datasets_root_path)
    extract_metafeatures_metalearningdb(datasets_root_path)



import logging
import pandas as pd
import datamart_profiler
from metalearn import Metafeatures

logger = logging.getLogger(__name__)

DEFAULT_METAFEATURES = [
    'NumberOfInstances', 'NumberOfFeatures', 'NumberOfNumericFeatures', 'NumberOfCategoricalFeatures',
    'RatioOfNumericFeatures', 'RatioOfCategoricalFeatures', 'NumberOfClasses', 'MeanClassProbability',
    'StdevClassProbability', 'SkewClassProbability', 'KurtosisClassProbability', 'MinClassProbability',
    'Quartile1ClassProbability', 'Quartile2ClassProbability', 'Quartile3ClassProbability', 'MaxClassProbability',
    'MinorityClassSize', 'MajorityClassSize', 'Dimensionality', 'NumberOfMissingValues', 'RatioOfMissingValues',
    'NumberOfInstancesWithMissingValues', 'RatioOfInstancesWithMissingValues', 'NumberOfFeaturesWithMissingValues',
    'RatioOfFeaturesWithMissingValues', 'MeanCardinalityOfCategoricalFeatures', 'StdevCardinalityOfCategoricalFeatures',
    'SkewCardinalityOfCategoricalFeatures', 'KurtosisCardinalityOfCategoricalFeatures',
    'MinCardinalityOfCategoricalFeatures', 'Quartile1CardinalityOfCategoricalFeatures',
    'Quartile2CardinalityOfCategoricalFeatures', 'Quartile3CardinalityOfCategoricalFeatures',
    'MaxCardinalityOfCategoricalFeatures', 'MeanCardinalityOfNumericFeatures',
    'StdevCardinalityOfNumericFeatures', 'SkewCardinalityOfNumericFeatures', 'KurtosisCardinalityOfNumericFeatures',
    'MinCardinalityOfNumericFeatures', 'Quartile1CardinalityOfNumericFeatures', 'Quartile2CardinalityOfNumericFeatures',
    'Quartile3CardinalityOfNumericFeatures', 'MaxCardinalityOfNumericFeatures'
]


def extract_metafeatures(dataset_path, target_column):
    data = pd.read_csv(dataset_path)
    Y = data[target_column]
    Y = pd.Series([str(i) for i in Y], name=target_column)  # Cast to string to get metalearn lib working correctly
    X = data.drop(columns=[target_column])
    metafeatures = Metafeatures()
    mfs = metafeatures.compute(X, Y, metafeature_ids=DEFAULT_METAFEATURES, seed=0, timeout=300)

    return mfs


def extract_dataprofiles(dataset_path, target_column, ignore_target_column=False):
    metadata = datamart_profiler.process_dataset(dataset_path, coverage=False)
    feature_types = set()
    missing_values = False
    for item in metadata['columns']:
        if ignore_target_column and item['name'] == target_column:
            continue
        identified_types = item['semantic_types'] if len(item['semantic_types']) > 0 else [item['structural_type']]
        for feature_type in identified_types:
            feature_types.add(feature_type)

        if 'missing_values_ratio' in item and item['name'] != target_column:
            missing_values = True

    dps = {'feature_types': sorted(feature_types), 'missing_values': missing_values}

    return dps
